@@ -11,6 +11,12 @@ regardless of their dimension alignments. Fills with zeros where necessary.
 
 '''
 
+# Python 3 does not have long, only int
+try:
+    long
+except NameError:
+    long = int
+
 from scipy import sparse
 import numpy as np
 
@@ -22,10 +28,11 @@ class csc_matrixPlus(sparse.csc_matrix):
         sparse.csc_matrix.__init__(self, arg1, shape, dtype, copy)
         if fromMatrix:
             self.__dict__.update(fromMatrix.__dict__)
-        from cylp.py.modeling import CyLPExpr as CyLPExpr 
+        from cylp.py.modeling import CyLPExpr as CyLPExpr
         self.CyLPExpr = CyLPExpr
+        self.rowScaleFactor = self.colScaleFactor = None
 
-    def __setitem__(self, (iRow, iCol), val):
+    def __setitem__(self, location, val):
         '''
         Set the item in row ``i`` and column ``j`` to ``val``.
         Increases matrix's size if necessary
@@ -45,9 +52,10 @@ class csc_matrixPlus(sparse.csc_matrix):
                 [ 2,  3,  6,  0,  0, 11]])
 
         '''
+        iRow, iCol = location
         if not isinstance(val, (int, long, float)):
             return sparse.csc_matrix.__setitem__(self, (iRow, iCol), val)
-            
+
         nCols = self.shape[1]
 
         #update shape if nec.
@@ -60,7 +68,7 @@ class csc_matrixPlus(sparse.csc_matrix):
                     self.data[i] = val
                     return
             #If we reach here it means that index does NOT exist
-            for i in xrange(iCol + 1, nCols + 1):
+            for i in range(iCol + 1, nCols + 1):
                 self.indptr[i] += 1
             indexOfElement = self.indptr[iCol + 1] - 1
             #If indices is empty
@@ -106,17 +114,75 @@ class csc_matrixPlus(sparse.csc_matrix):
         a = np.array(nCol * [nElement], dtype=np.int32)
         self.indptr = np.concatenate((self.indptr, a), axis=0)
         self._shape = (self._shape[0], self.shape[1] + nCol)
-    
+
     def __getitem__(self, key):
         ret = sparse.csc_matrix.__getitem__(self, key)
         if isinstance(ret, (int, long, float)):
             return ret
+        # This seems to cause some potential problems when the result is 1x1
+        # It should really be returned as an int/float in that case, but
+        # this prevents it and causes behavior to be different than
+        # sparse.csr_matrix
         return csc_matrixPlus(ret)
-    
-    @property 
+
+    def row_scale(self, scaleFactor=None):
+        data = self.data
+        m = self.tocoo()
+        irow, jcol = m.row, m.col
+        if scaleFactor is not None:
+            data /= scaleFactor[irow]
+            self.rowScaleFactor = scaleFactor
+            return scaleFactor
+
+        rowScaleFactor = np.zeros(self.shape[0], dtype=np.float)
+        for k in range(len(data)):
+            row = irow[k]
+            val = abs(data[k])
+            rowScaleFactor[row] = max(rowScaleFactor[row], val)
+        rowScaleFactor[rowScaleFactor == 0.0] = 1.0
+        data /= rowScaleFactor[irow]
+        self.rowScaleFactor = rowScaleFactor
+        return rowScaleFactor
+
+
+    def col_scale(self, scaleFactor=None):
+        data = self.data
+        m = self.tocoo()
+        irow, jcol = m.row, m.col
+        if scaleFactor is not None:
+            data /= scaleFactor[jcol]
+            self.colScaleFactor = scaleFactor
+            return scaleFactor
+
+        colScaleFactor = np.zeros(self.shape[1], dtype=np.float)
+
+        for k in range(len(data)):
+            col = jcol[k]
+            val = abs(data[k])
+            colScaleFactor[col] = max(colScaleFactor[col], val)
+        colScaleFactor[colScaleFactor == 0.0] = 1.0
+        data /= colScaleFactor[jcol]
+        self.colScaleFactor = colScaleFactor
+        return colScaleFactor
+
+    def col_unscale(self, scaleFactor=None):
+        if scaleFactor is None:
+            scaleFactor = self.colScaleFactor
+        if scaleFactor is not None:
+            jcol = self.tocoo().col
+            self.data *= scaleFactor[jcol]
+
+    def row_unscale(self, scaleFactor=None):
+        if scaleFactor is None:
+            scaleFactor = self.rowScaleFactor
+        if scaleFactor is not None:
+            irow = self.tocoo().row
+            self.data *= scaleFactor[irow]
+
+    @property
     def T(self):
         return csr_matrixPlus(sparse.csc_matrix.transpose(self))
-    
+
     def __le__(self, other):
         if isinstance(other, self.CyLPExpr):
             return NotImplemented
@@ -174,10 +240,11 @@ class csr_matrixPlus(sparse.csr_matrix):
         sparse.csr_matrix.__init__(self, arg1, shape, dtype, copy)
         if fromMatrix:
             self.__dict__.update(fromMatrix.__dict__)
-        from cylp.py.modeling import CyLPExpr as CyLPExpr 
+        from cylp.py.modeling import CyLPExpr as CyLPExpr
         self.CyLPExpr = CyLPExpr
+        self.rowScaleFactor = self.colScaleFactor = None
 
-    def __setitem__(self, (iRow, iCol), val):
+    def __setitem__(self, location, val):
         '''
         Sets the item in row ``i`` and col ``j`` to ``val``.
         Increases matrix's ``shape[1]`` if necessary
@@ -191,7 +258,7 @@ class csr_matrixPlus(sparse.csr_matrix):
         >>> data = np.array([1, 2, 3, 4, 5, 6])
         >>> s = csr_matrixPlus((data, indices, indptr), shape=(3, 3))
         >>> s[5,2] = 11
-        >>> print s.todense()
+        >>> print(s.todense())
         [[ 1  0  2]
          [ 0  0  3]
          [ 4  5  6]
@@ -200,18 +267,19 @@ class csr_matrixPlus(sparse.csr_matrix):
          [ 0  0 11]]
 
         '''
+        iRow, iCol = location
         if not isinstance(val, (int, long, float)):
             return sparse.csr_matrix.__setitem__(self, (iRow, iCol), val)
-        
+
         nRows = self.shape[0]
         nCols = self.shape[1]
-        
+
         l = self.tolil()
-        
+
         if iCol >= nCols:
             l._shape = (nRows, iCol + 1)
             nCols = iCol + 1
-        
+
         if iRow >= nRows:
             l._shape = (iRow + 1, nCols)
             nRowsToAdd = iRow + 1 - nRows
@@ -221,7 +289,7 @@ class csr_matrixPlus(sparse.csr_matrix):
 
         l[iRow, iCol] = val
         s = csr_matrixPlus(l)
-        
+
         self._nnz = s.nnz
         self._shape = s._shape
         self.indices = s.indices
@@ -241,7 +309,7 @@ class csr_matrixPlus(sparse.csr_matrix):
                     self.data[i] = val
                     return
             #if we reach here it means that index does NOT exist
-            for i in xrange(iRow + 1, nRows + 1):
+            for i in range(iRow + 1, nRows + 1):
                 self.indptr[i] += 1
             indexOfElement = self.indptr[iRow + 1] - 1
             # If indices is empty
@@ -290,12 +358,71 @@ class csr_matrixPlus(sparse.csr_matrix):
         ret = sparse.csr_matrix.__getitem__(self, key)
         if isinstance(ret, (int, long, float)):
             return ret
+        # This seems to cause some potential problems when the result is 1x1
+        # It should really be returned as an int/float in that case, but
+        # this prevents it and causes behavior to be different than
+        # sparse.csr_matrix
         return csr_matrixPlus(ret)
 
-    @property 
+    def row_scale(self, scaleFactor=None):
+        data = self.data
+        m = self.tocoo()
+        irow, jcol = m.row, m.col
+        if scaleFactor is not None:
+            data /= scaleFactor[irow]
+            self.rowScaleFactor = scaleFactor
+            return scaleFactor
+
+        rowScaleFactor = np.zeros(self.shape[0], dtype=np.float)
+        for k in range(len(data)):
+            row = irow[k]
+            val = abs(data[k])
+            rowScaleFactor[row] = max(rowScaleFactor[row], val)
+        rowScaleFactor[rowScaleFactor == 0.0] = 1.0
+        data /= rowScaleFactor[irow]
+        self.rowScaleFactor = rowScaleFactor
+        return rowScaleFactor
+
+
+    def col_scale(self, scaleFactor=None):
+        data = self.data
+        m = self.tocoo()
+        irow, jcol = m.row, m.col
+        if scaleFactor is not None:
+            data /= scaleFactor[jcol]
+            self.colScaleFactor = scaleFactor
+            return scaleFactor
+
+        colScaleFactor = np.zeros(self.shape[1], dtype=np.float)
+
+        for k in range(len(data)):
+            col = jcol[k]
+            val = abs(data[k])
+            colScaleFactor[col] = max(colScaleFactor[col], val)
+        colScaleFactor[colScaleFactor == 0.0] = 1.0
+        data /= colScaleFactor[jcol]
+        self.colScaleFactor = colScaleFactor
+        return colScaleFactor
+
+    def col_unscale(self, scaleFactor=None):
+        if scaleFactor is None:
+            scaleFactor = self.colScaleFactor
+        if scaleFactor is not None:
+            jcol = self.tocoo().col
+            self.data *= scaleFactor[jcol]
+
+    def row_unscale(self, scaleFactor=None):
+        if scaleFactor is None:
+            scaleFactor = self.rowScaleFactor
+        if scaleFactor is not None:
+            irow = self.tocoo().row
+            self.data *= scaleFactor[irow]
+
+
+    @property
     def T(self):
         return csc_matrixPlus(sparse.csr_matrix.transpose(self))
-    
+
     def __le__(self, other):
         if isinstance(other, self.CyLPExpr):
             return NotImplemented
@@ -351,9 +478,9 @@ def sparseConcat(a, b, how, v_offset=0, h_offset=0):
     Concatenate two sparse matrices, ``a`` and ``b``, horizontally if
     ``how = 'h'``, and vertically if ``how = 'v'``.
     Add zero rows and columns if dimensions don't align.
-    ``v_offset`` specifies how to align ``b`` along side ``a``. The 
+    ``v_offset`` specifies how to align ``b`` along side ``a``. The
     value of ``v_offset`` will be added to each row index of ``b``.
-    ``v_offset=-1`` means that we want the greatest possible offset without 
+    ``v_offset=-1`` means that we want the greatest possible offset without
     changeing the dimensions.
     ``h_offset`` is a similar argument but to specify horizontal offset.
 
@@ -384,11 +511,11 @@ def sparseConcat(a, b, how, v_offset=0, h_offset=0):
     True
 
     '''
-    if a == None:
-        if b == None:
+    if a is None:
+        if b is None:
             return None
         return csr_matrixPlus(b)
-    if b == None:
+    if b is None:
         return csr_matrixPlus(a)
     assert(h_offset >= -1 and v_offset >= -1)
 
@@ -415,7 +542,7 @@ def sparseConcat(a, b, how, v_offset=0, h_offset=0):
         if h_offset == -1:
             assert(a.shape[1] > b.shape[1])
             h_offset = a.shape[1] - b.shape[1]
-        
+
         assert(v_offset >= 0)
 
         row = np.concatenate((a.row, b.row + (a.shape[0] + v_offset)), axis=0)
